//-----------------------------------------------------------------------------
// F41x_UART_STDIO.c
//-----------------------------------------------------------------------------
// Copyright 2008 Silicon Laboratories, Inc.
// http://www.silabs.com
//
// Program Description:
//
// This program demonstrates how to configure the C8051F410 to use routines
// in STDIO.h to write to and read from the UART interface.  The program
// reads a character using the STDIO routine getkey(), outputs that character
// to the screen, and then outputs the ASCII hex value of that character.
//
//
// How To Test:
//
// 1) Ensure that jumpers are placed on J27 of the C8051F410 target board
//    that connect the P0.4 pin to the TX signal, and P0.5 to the RX signal.
// 2) Ensure that the serial cable is connected to the RS232 connector
//    on the target board.
// 3) Specify the target baudrate in the constant <BAUDRATE>.
// 4) Open Hyperterminal, or a similar program, and connect to the target
//    board's serial port. 
// 5) Download and execute code on an 'F41x target board.
//   
//
// Target:         C8051F41x
// Tool chain:     Keil C51 7.50 / Keil EVAL C51
// Command Line:   None
//
// Release 1.1 / 12 MAR 2008 (AS)
//    -Removed nonexistent function prototype and other formatting edits
//
// Release 1.0 / 17 AUG 2006 (PD)
//    -Initial Revision
//

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "c8051f410.h"                 // SFR declarations
#include <stdio.h>

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------
#define SYSCLK      24500000           // SYSCLK frequency in Hz
#define BAUDRATE       19200           // Baud rate of UART in bps

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
static float	f_pwm_duty = 0.5;	   	// PWM duty cycle 97kHz
									   	// ex) 50% => f_pwm_duty = 0.5;

static long		cnt = 0;			   	// increment counter
static long		ad_value [4]={0,0,0,0};	// a/d value
static char		in_value [2]={0,0};	   	// input I/O value
static char		out_value[2]={0,0};	   	// output I/O value

//-----------------------------------------------------------------------------
// Function PROTOTYPES
//-----------------------------------------------------------------------------
void SYSCLK_Init (void);
void UART0_Init (void);
void PORT_Init (void);
void ADC0_Init (void);
long ad_read(unsigned char channel);
void pwm_write(float duty);
void wait(long t);

//-----------------------------------------------------------------------------
// MAIN Routine
//-----------------------------------------------------------------------------
void main (void) 
{
   extern float f_pwm_duty;

   PCA0MD &= ~0x40;                    // WDTE = 0 (clear watchdog timer 
                                       // enable)
   PORT_Init();                        // Initialize Port I/O
   SYSCLK_Init ();                     // Initialize Oscillator
   UART0_Init();
   PCA0_Init ();                       // Initialize PCA0
   ADC0_Init();                        // Initialize ADC0
   EA = 1;                             // Enable global interrupts

   // Main Loop
   while (1)
   {
		// 1. getting parmeter
		//---------------------------------------------------------------------
		ad_value[0] = ad_read(0);	wait(10000);	// P1.0
		ad_value[1] = ad_read(1);	wait(10000);	// P1.1
		ad_value[2] = ad_read(3);	wait(10000);	// P1.3
		ad_value[3] = ad_read(4);	wait(10000);	// P1.4

		in_value[0] = P2_0;
		in_value[1] = P2_1;

		// 2. calculation
		//---------------------------------------------------------------------
		// TODO:

		// 3. output AB/PWM
		// ex: DUTY 50% => f_pwm_duty = 0.5;
		//---------------------------------------------------------------------
		P2_4 = out_value[0];	// P2.4
		P2_5 = out_value[1];	// P2.5

		f_pwm_duty = 0.65;
		pwm_write(f_pwm_duty);

		// 4. output debug 19200bps/non/8/1
		//---------------------------------------------------------------------
		putchar(cnt);
		putchar(in_value[0]);
		putchar(in_value[1]);
	  	putchar((unsigned char)(ad_value[0] >> 8));		// P1.0
	  	putchar((unsigned char)(ad_value[0] & 0x00FF));
		putchar((unsigned char)(ad_value[1] >> 8));		// P1.1
	  	putchar((unsigned char)(ad_value[1] & 0x00FF));
	  	putchar((unsigned char)(ad_value[2] >> 8));		// P1.3
	  	putchar((unsigned char)(ad_value[2] & 0x00FF));
	  	putchar((unsigned char)(ad_value[3] >> 8));		// P1.4
	  	putchar((unsigned char)(ad_value[3] & 0x00FF));

	  	putchar(0x0D);	// return code
  	  	putchar(0x0A);	// return code

		// 5. increment
		//---------------------------------------------------------------------
		cnt++;
   }
}

//-----------------------------------------------------------------------------
// Initialization Subroutines
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// PORT_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Configure the Crossbar and GPIO ports.
//
// P0.0  -  digital,     Open-Drain, PWM
// P0.1  -  Skipped,     Open-Drain, Digital
// P0.2  -  Skipped,     Open-Drain, Digital
// P0.3  -  Skipped,     Open-Drain, Digital
// P0.4  -  digital,	 push-pull , UART TX
// P0.5  -  digital,     open-drain, UART RX
// P0.6  -  Skipped,     Open-Drain, Digital
// P0.7  -  Skipped,     Open-Drain, Digital

// P1.0  -  Skipped,     Open-Drain, Analog
// P1.1  -  Skipped,     Open-Drain, Analog
// P1.2  -  Skipped,     Open-Drain, Analog
// P1.3  -  Skipped,     Open-Drain, Analog
// P1.4  -  Skipped,     Open-Drain, Analog
// P1.5  -  Skipped,     Open-Drain, Analog
// P1.6  -  Skipped,     Open-Drain, Analog
// P1.7  -  Skipped,     Open-Drain, Analog

// P2.0  -  Skipped,     Open-Drain, Digital
// P2.1  -  Skipped,     Open-Drain, Digital
// P2.2  -  Skipped,     Open-Drain, Digital
// P2.3  -  Skipped,     Open-Drain, Digital
// P2.4  -  Skipped,     push-pull , Digital
// P2.5  -  Skipped,     push-pull , Digital
// P2.6  -  Skipped,     push-pull , Digital
// P2.7  -  Skipped,     push-pull , Digital
// 
//-----------------------------------------------------------------------------
void PORT_Init (void)
{
   XBR0    = 0x01;                     // Enable UART on P0.4(TX) and P0.5(RX)                     
   XBR1    = 0x41;                     // Route CEX0 to P0.0,
                                       // Enable crossbar and weak pull-ups

   P0MDOUT |= 0x01;                    // Set CEX0 (P0.0) to push-pull
   P1SKIP = 0x7F;                      // Skip all analog pins
   P1MDOUT	= 0x80;
   P1MDIN = 0x00;                      // Set desired pins as analog inputs
   P2MDOUT  = 0xF0;
}

//-----------------------------------------------------------------------------
// SYSCLK_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// This routine initializes the system clock to use the internal oscillator
// at its maximum frequency.
// Also enables the Missing Clock Detector.
//-----------------------------------------------------------------------------
void SYSCLK_Init (void)
{
   OSCICN = 0x87;                      // configure internal oscillator for
                                       // 24.5MHz 
   RSTSRC = 0x04;                      // enable missing clock detector
}

//-----------------------------------------------------------------------------
// UART0_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Configure the UART0 using Timer1, for <BAUDRATE> and 8-N-1.
//-----------------------------------------------------------------------------
void UART0_Init (void)
{
   SCON0 = 0x10;                       // SCON0: 8-bit variable bit rate
                                       //        level of STOP bit is ignored
                                       //        RX enabled
                                       //        ninth bits are zeros
                                       //        clear RI0 and TI0 bits
   if (SYSCLK/BAUDRATE/2/256 < 1) {
      TH1 = -(SYSCLK/BAUDRATE/2);
      CKCON |=  0x08;                  // T1M = 1; SCA1:0 = xx
   } else if (SYSCLK/BAUDRATE/2/256 < 4) {
      TH1 = -(SYSCLK/BAUDRATE/2/4);
      CKCON &= ~0x0B;                  // T1M = 0; SCA1:0 = 01
      CKCON |=  0x01;
   } else if (SYSCLK/BAUDRATE/2/256 < 12) {
      TH1 = -(SYSCLK/BAUDRATE/2/12);
      CKCON &= ~0x0B;                  // T1M = 0; SCA1:0 = 00
   } else if (SYSCLK/BAUDRATE/2/256 < 48) {
      TH1 = -(SYSCLK/BAUDRATE/2/48);
      CKCON &= ~0x0B;                  // T1M = 0; SCA1:0 = 10
      CKCON |=  0x02;
   } else {
      while (1);                       // Error.  Unsupported baud rate
   }

   TL1 = TH1;                          // init Timer1
   TMOD &= ~0xf0;                      // TMOD: timer 1 in 8-bit autoreload
   TMOD |=  0x20;
   TR1 = 1;                            // START Timer1
   TI0 = 1;                            // Indicate TX0 ready
}

//-----------------------------------------------------------------------------
// putchar
//-----------------------------------------------------------------------------
void putchar(char c) 
{ 
 	while(!TI);  
 	TI=0; 
 	SBUF = c; 
}

//-----------------------------------------------------------------------------
// PCA0_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// This function configures the PCA time base, and sets up 8-bit PWM output
// mode for Module 0 (CEX0 pin).
//
// The frequency of the PWM signal generated at the CEX0 pin is equal to the
// PCA main timebase frequency divided by 256.
//
// The PCA time base in this example is configured to use SYSCLK, and SYSCLK
// is set up to use the internal oscillator running at 24.5 MHz.  Therefore,
// the frequency of the PWM signal will be 24.5 MHz / 256 = 95.7 kHz.
// Using different PCA clock sources or a different processor clock will
// result in a different frequency for the PWM signal.
//
//    -------------------------------------------------------------------------
//    How "8-Bit PWM Mode" Works:
//
//       The PCA's 8-bit PWM Mode works by setting an output pin low every
//    time the main PCA counter low byte (PCA0L) overflows, and then setting
//    the pin high whenever a specific match condition is met.
//
//    Upon a PCA0L overflow (PCA0L incrementing from 0xFF to 0x00), two things
//    happen:
//
//    1) The CEXn pin will be set low.
//    2) The contents of the PCA0CPHn register for the module are copied into
//       the PCA0CPLn register for the module.
//
//    When the PCA0L register increments and matches the PCA0CPLn register for
//    the selected module, the CEXn pin will be set high, except when the
//    ECOMn bit in PCA0CPMn is cleared to '0'.  By varying the value of the
//    PCA0CPHn register, the duty cycle of the waveform can also be varied.
//
//    When ECOMn = '1', the duty cycle of the PWM waveform is:
//
//       8-bit PWM Duty Cycle = (256 - PCA0CPLn) / 256
//
//    To set the duty cycle to 100%, a value of 0x00 should be loaded into the
//    PCA0CPHn register for the module being used (with ECOMn set to '1').
//    When the value of PCA0CPLn is equal to 0x00, the pin will never be
//    set low.
//
//    To set the duty cycle to 0%, the ECOMn bit in the PCA0CPMn register
//    should be cleared to 0.  This prevents the PCA0CPLn match from occuring,
//    which results in the pin never being set high.
//    -------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void PCA0_Init (void)
{
	extern float f_pwm_duty;

	// Configure PCA time base; overflow interrupt disabled
	PCA0CN = 0x00;                      // Stop counter; clear all flags
	PCA0MD = 0x08;                      // Use SYSCLK as time base

	PCA0CPM0 = 0x42;                    // Module 0 = 8-bit PWM mode

 	// Configure initial PWM duty cycle
 	if(f_pwm_duty > 1) f_pwm_duty = 1;
   	if(f_pwm_duty < 0) f_pwm_duty = 0;

 	PCA0CPH0 = 256 - (unsigned char)(256 * f_pwm_duty);

 	// Start PCA counter
 	CR = 1;
}

//-----------------------------------------------------------------------------
// ADC0_Init
//-----------------------------------------------------------------------------
//
// Return Value : None
// Parameters   : None
//
// Configures ADC0 to make single-ended analog measurements on pin P1.1
//
//-----------------------------------------------------------------------------
void ADC0_Init (void)
{
   ADC0CN = 0x00;                      // ADC0 conversion initiated on every write of e1f to AD0BUSY.

   REF0CN = 0x0B;               	   // VDD used as voltage reference.

   ADC0MX = 0x09;					   // Set P1.1 as positive input	

   ADC0CF = ((SYSCLK/3000000)-1)<<3;   // set SAR clock to 3MHz

   ADC0CF |= 0x00;                     // right-justify results 

   EIE1 |= 0x08;                       // enable ADC0 conversion complete int.

   AD0EN = 1;                          // enable ADC0
}

//-----------------------------------------------------------------------------
// ADC read command
//-----------------------------------------------------------------------------
long ad_read(unsigned char channel)
{
	long temp = 0x00;

	switch(channel)
	{
		// select ADC channel
		case 0:	ADC0MX = 0x08;	break;	// P1.0
		case 1:	ADC0MX = 0x09;	break;	// P1.1
		case 2:	ADC0MX = 0x0A;	break;	// P1.2
		case 3:	ADC0MX = 0x0B;	break;	// P1.3
		case 4:	ADC0MX = 0x0C;	break;	// P1.4
		case 5:	ADC0MX = 0x0D;	break;	// P1.5
		case 6:	ADC0MX = 0x0E;	break;	// P1.6
		case 7:	ADC0MX = 0x0F;	break;	// P1.7

		default :
			ADC0MX = 0x08;	break;		// P1.0
	}

	AD0BUSY = 1;						// Start A/D C!!
	while(!AD0INT);						// wait complete ADC
 		
	temp = ADC0;						// Get ADC
	AD0INT = 0;							// clear ADC0 conv. complete flag

	return temp;
}

//-----------------------------------------------------------------------------
// PWM write command
//-----------------------------------------------------------------------------
void pwm_write(float duty)
{
 	// Configure initial PWM duty cycle
 	if(duty > 1) duty = 1;
   	if(duty < 0) duty = 0;

 	PCA0CPH0 = 256 - (unsigned char)(256 * duty);
}

//-----------------------------------------------------------------------------
// wait
//-----------------------------------------------------------------------------
void wait(long t)
{
	while(t--);
}

//-----------------------------------------------------------------------------
// End Of File
//-----------------------------------------------------------------------------